/*
 * Copyright (c) 1989 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 */

#ifndef lint
static char copyright[] = "Copyright (c) 1990 Regents of the University of California.\nAll rights reserved.\n";
static char SccsId[] = "@(#)@(#)pop_get_command.c	2.1  2.1 3/18/91";
#endif not lint

/* Popper help added July 1993, by Thomas R. Kimpton. */


#include <stdio.h>
#include <sys/types.h>

#ifdef SOLARIS2
#include <string.h>
#else
#include <strings.h>
#endif

#include "popper.h"

/* 
 *  get_command:    Extract the command from an input line form a POP client
 */

state_table states[] = {
        auth1,  "user", 1,  1,  pop_user,   {auth1, auth2},
		"user login:    Here is the login name of the user account.\n",
        auth2,  "pass", 1,  1,  pop_pass,   {auth1, trans},
		"pass password: Here is the password of the user account.\n",
#ifdef RPOP
        auth2,  "rpop", 1,  1,  pop_rpop,   {auth1, trans},
#endif RPOP
        auth1,  "quit", 0,  0,  pop_quit,   {halt,  halt},
		"quit:          End this POP session.\n",
        auth2,  "quit", 0,  0,  pop_quit,   {halt,  halt},
		"",
        trans,  "stat", 0,  0,  pop_stat,   {trans, trans},
		"stat:          Send number of messages, and total size.\n",
        trans,  "list", 0,  1,  pop_list,   {trans, trans},
		"list [id]:     Send number of messages, and total size, and send a list of\n               message ID's with each message's size. Optionally send the\n               id and size of message \"id\".\n",
        trans,  "retr", 1,  1,  pop_send,   {trans, trans},
		"retr id:       Retrieve message \"id\" from the POP server.\n",
        trans,  "dele", 1,  1,  pop_dele,   {trans, trans},
		"dele id:       Mark message \"id\" for deletion from the POP server.\n",
        trans,  "noop", 0,  0,  NULL,       {trans, trans},
		"noop:          No operation.\n",
        trans,  "rset", 0,  0,  pop_rset,   {trans, trans},
		"rset:          Reset messages marked for deletion to unmarked, reset\n               \"highest message id\" to initial value.\n",
        trans,  "top",  2,  2,  pop_send,   {trans, trans},
		"top id n:      Send top \"n\" lines of message \"id\".\n",
        trans,  "last", 0,  0,  pop_last,   {trans, trans},
		"last:          Send id of highest message accessed.\n",
        trans,  "xtnd", 1,  99, pop_xtnd,   {trans, trans},
		"xtnd selector [arg ...]:\n               POP extended command, currently only the \"xmit\" selector is\n               supported.\n",
        trans,  "quit", 0,  0,  pop_updt,   {halt,  halt},
		"",
        trans,  "help", 0,  1,  pop_help,   {trans,  trans},
		"help [topic]:  Send the help text, optionally send help on \"topic.\"\n",
        auth1,  "help", 0,  1,  pop_help,   {auth1,  auth1},
		"",
        auth2,  "help", 0,  1,  pop_help,   {auth2,  auth2},
		"",
        (state) 0,  NULL,   0,  0,  NULL,       {halt,  halt},
};

state_table *pop_get_command(p,mp)
POP             *   p;
register char   *   mp;         /*  Pointer to unparsed line 
                                    received from the client */
{
    state_table     *   s;
    char                buf[MAXMSGLINELEN];

    /*  Save a copy of the original client line */
#ifdef DEBUG
    if(p->debug) strcpy (buf,mp);
#endif DEBUG

    /*  Parse the message into the parameter array */
    if ((p->parm_count = pop_parse(p,mp)) < 0) return(NULL);

    /*  Do not log cleartext passwords */
#ifdef DEBUG
    if(p->debug){
        if(strcmp(p->pop_command,"pass") == 0)
            pop_log(p,POP_DEBUG,"Received: \"%s xxxxxxxxx\"",p->pop_command);
        else {
            /*  Remove trailing <LF> */
            buf[strlen(buf)-2] = '\0';
            pop_log(p,POP_DEBUG,"Received: \"%s\"",buf);
        }
    }
#endif DEBUG

    /*  Search for the POP command in the command/state table */
    for (s = states; s->command; s++) {

        /*  Is this a valid command for the current operating state? */
        if (strcmp(s->command,p->pop_command) == 0
             && s->ValidCurrentState == p->CurrentState) {

            /*  Were too few parameters passed to the command? */
            if (p->parm_count < s->min_parms)
                return((state_table *)pop_msg(p,POP_FAILURE,
                    "Too few arguments for the %s command.",p->pop_command));

            /*  Were too many parameters passed to the command? */
            if (p->parm_count > s->max_parms)
                return((state_table *)pop_msg(p,POP_FAILURE,
                    "Too many arguments for the %s command.",p->pop_command));

            /*  Return a pointer to the entry for this command in 
                the command/state table */
            return (s);
        }
    }
    /*  The client command was not located in the command/state table */
    return((state_table *)pop_msg(p,POP_FAILURE,
        "Unknown command: \"%s\".",p->pop_command));
}
